

function [vfz,coeff,A]=Min_quad(x,y,z,fb)

%__________________________________________________________________________
%
% Funzione: Min_quad
% ___________________
%
% Chiamata: [vfz,coeff]=Min_quad(x,y,z,fb); se si desidera anche il residuo: 
%           [vfz,coeff,A]=Min_quad(x,y,z,fb)
%
% La function 'Min_quad' valuta la funzione approssimante nel senso dei
% minimi quadrati in punti assegnati z, costruita utilizzando m punti 
% (x(i),y(i)), i=1,...,m e n funzioni base fb(j), j=1,...,n con n<=m.
% La funzione approssimante e' una combinazione lineare delle n 
% funzioni base; i coefficienti di tale combinazione vengono calcolati
% anch'essi dalla function.
%
% Parametri di input:
% ___________________
%
% x    Matrice di dimensione m x lcol, m,lcol>=1, contenente i valori
%      delle ascisse con cui si vuole costruire la funzione 
%      approssimante; se la funzione approssimante e' nel piano allora
%      lcol=1, se la funzione approssimante e' nello spazio, lcol=2;
%
% y    vettore colonna di lunghezza m contenente i valori delle
%      ordinate con cui si vuole costruire la funzione approssimante;
%
% z    Matrice  di dimensione k x lcol, k,lcol>=1, contenente, in ogni 
%      riga, il punto in cui si vuole valutare la funzione approssimante.
%
% fb   vettore colonna di lunghezza n le cui componenti fb(j),j=1,...,n
%      sono stringhe contenenti le espressioni, nella variabile t, di
%      n funzioni base. Si osservi che nel caso in cui il numero di 
%      colonne lcol di x, e' maggiore di 1, t diventa un vettore: ad 
%      esempio, se lcol=2, allora t=[t(1);t(2)].
%      Per costruire il vettore fb si puo' utilizzare il comando
%      'strvcat' nel seguente modo: fb=strvcat(fb1,...,fbn) dove 
%      fb1,...,fbn sono le stringhe componenti di fb.
%
%
% Parametri di output:
% ____________________
%
% vfz    Matrice di dimensione k x (lcol+1) contenente, sulla riga 
%        j_esima, il punto z(j) (di dimensione lcol) in cui e' valutata
%        la funzione approssimante e la valutazione della funzione 
%        stessa in z(j); 
%
% coeff  vettore colonna, di lunghezza n, la cui componente j_esima
%        contiene il valore del coefficiente che moltiplica la j_esima
%        funzione base fb(j), nella funzione approssimante rappresentata
%        come c1*fb(1)+...+cn*fb(n).
%
% A      matrice di dimensione n x m, la cui j_esima colonna contiene i
%        valori assunti dalla j_esima funzione base nei punti x(i), 
%        i=1,...,n. 
%_________________________________________________________________________
%
%
% 1 esempio di utilizzo della function:
% ___________________________________
%
% clear all
% format short e
%
% x=[1 1 1 1 1 2 3 4 5 5 5]';
% y=[1.1 1.05 1.12 1.07 2 2.1 3.05 3.7 4.0 4.8 5.2]';
% z=sort([x;0.5;2.6; 5.1]);
% fb=strvcat('1','t');
%
% 
% [vfz,coeff]=Min_quad(x,y,z,fb)
% 
%
% % Se si desidera conoscere il residuo si puo' chiamare la function
% % inserendo un ulteriore parametro di output, che e' la matrice A
% % e sostituendo la chiamata precedente con:
% 
% [vfz,coeff,A]=Min_quad(x,y,z,fb)
% 
% 
%
% 2 esempio di utilizzo della function (in 3D):
% _____________________________________________
%
% x1=[0.0 1.2 2.1 3.4 4.0 4.2 5.6 5.8 6.9]';
% x2=[0.0 0.5 6.0 0.5 5.1 3.2 1.3 7.4 10.2]';
% x=[x1 x2];
% y=[1.2 3.4 -4.5 9.9 2.4 7.2 14.3 3.5 1.2]';
% z=x;
%
% % Le funzioni base sono: 1,x1,x2
%
% fb=strvcat('1','t(1)','t(2)');
% 
% [vfz,coeff,A]=Min_quad(x,y,z,fb)
% 
%__________________________________________________________________________




[k,l]=size(z);
[m,l]=size(x);
[n,s]=size(fb);

for i=1:m
    t=x(i,:);
    for j=1:n
        A(i,j)=eval(fb(j,:));
    end
end
   
if rank(A)==min(m,n)
    
    % Decomposizione QR
    
    [Q,R]=qr(A,0);
    coeff=R\(R'\(A'*y));
    r=y-A*coeff;
    e=R\(R'\(A'*r));
    coeff=coeff+e;
else
    
    % Decomposizione SVD

    [U,S,V]=svd(A,0);
    D=S'*S;
    r=rank(D);
    d=1./diag(D(1:r,1:r));
    Dr=diag([d;zeros(n-r,1)]);
    coeff=(V*Dr*V')*(A'*y);
    
end

for i=1:k
    t=z(i,:);
    for j=1:n
        Az(i,j)=eval(fb(j,:));
    end
end
   

vfz=Az*coeff;
vfz=[z vfz];




