
function [P,stimaerr]=Neville(x,y,z,ind,np)

%___________________________________________________________________________
%
% Funzione:  Neville
%___________________
%
% Chiamata:  P=Neville(x,y,ind,z,np); se si desidera conoscere
%            anche una stima dell'errore commesso:
%
%            [P,stimaerr]=Neville(x,y,z,ind,np);
%
% La function 'Neville' calcola i valori assunti nei punti z(k), 
% k=1,...,m, m>=1, dati in ingresso, dal polinomio interpolante di 
% Neville, di grado np, np>=0, costruito su np+1 punti  degli n+1 
% (x(i),y(i)), i=1,...,n+1, con np<=n, anch'essi dati in ingresso.   
% 
%
% Parametri di input:
%____________________
%
% x,y     Vettori colonna di lunghezza n+1, contenenti, rispettivamente
%         le ascisse (nodi), che devono essere distinte, ossia x(i) deve
%         essere diverso da x(j) se i e' diverso da j,e le ordinate su 
%         cui si vuole costruire il polinomio interpolante di grado np.
%
% z       Vettore colonna di lunghezza m, contenente i punti in cui si 
%         vuole valutare il polinomio interpolante. 
%
% ind     Vettore colonna di interi, di lunghezza lind+1, con lind <= n,
%         contenente la posizione, nel vettore x, dei nodi che si scelgono 
%         per costruire la tabella di Neville.
%
% np      variabile intera contenente il grado del polinomio che si 
%         vuole costruire; np deve essere inferiore alla lunghezza
%         del vettore  ind. 
% 
%
% Parametri di output:
%_____________________
%
% P       Matrice di dimensione m x 2, contenente, nella riga k-esima,il
%         punto z(k) e il valore assunto dal polinomio interpolante nel 
%         punto z(k).
%
% stimaerr vettore colonna di lunghezza m, la cui k-esima componente
%          contiene la stima dell'errore commesso in z(k) nella 
%          interpolazione dei dati con il polinomio interpolante di 
%          grado np. Tale output si puo' ottenere se np<lind.  
%___________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% clear all
% format short e
% 
% x=[0:0.5:2*pi]';
% t=x;
% f='sin(t)';
% y=eval(f);
% 
% z=[x ; -0.4; 1.3;0.54; 0.45;pi/2; pi;3*pi/2];
% z=sort(z);
% np=5;
% ind=[2 3 5 7 8 9 12]';
% 
% [P,stimaerr]=Neville(x,y,z,ind,np);
%___________________________________________________________________________


lind=length(ind)-1;
n=length(x)-1;
m=length(z);
P=[];stimaerr=[];

if lind>n
    disp('Il numero dei nodi scelti deve essere <= del numero dei nodi totali')
    return    
elseif np>lind
    disp('Il grado del polinomio deve essere minore del numero dei nodi scelti')
    return
end

for i=1:n
    for j=i+1:n+1
        if abs(x(i)-x(j))<=1.e-14
            disp('I nodi devono essere distinti')
            return
        end
    end
end

xy=sortrows([x y]);x=xy(:,1);y=xy(:,2);
xy=sortrows([x(ind) y(ind)]);x=xy(:,1);y=xy(:,2);

if lind>np
    xnp1=x(np+1);ynp1=y(np+1);
    x(np+1)=x(end);y(np+1)=y(end);
    x(end)=xnp1;y(end)=ynp1;
end

% Costruzione della tabella di Neville

   p=y(1:lind+1)*ones(1,m);
   for k=1:lind+1
      for i=lind+1:-1:k+1
         p(i,:)=p(i,:)+(z'-x(i)).*(p(i,:)-p(i-1,:))/(x(i)-x(i-k));
       end
   end
   P=[z, p(np+1,:)'];

   if lind>np
        stimaerr=abs(p(np+1,:)-p(np+2,:))';
   end   



