

function S=Spline_interp(x,y,z,p)

%__________________________________________________________________________
%
% Funzione:  Spline_interp
% ________________________
%
% Chiamata:   S=Spline_interp(x,y,z,p);
%
% La function 'spline_interp' calcola i valori assunti nei punti z(k), 
% k=1,...,m, m>=1, dati in ingresso, dalla funzione spline cubica 
% interpolante, costruita su n sottointervalli [x(i),x(i+1)], 
% i=1,...,n che partizionano l'intervallo [x(1),x(n+1)].  
% 
%
% Parametri di input:
%____________________
%
% x,y     Vettori colonna di lunghezza n+1, contenenti, rispettivamente
%         le ascisse (nodi), che devono essere distinte, ossia x(i) deve
%         essere diverso da x(j) se i e' diverso da j,e le ordinate su 
%         cui si vuole costruire la spline cubica. Il vettore x determina
%         la partizione dell'intervallo in cui si desidera interpolare
%         i dati.
%
% z       Vettore colonna di lunghezza m, contenente i punti in cui si 
%         vuole valutare la spline cubica. 
%
% p       vettore riga di lunghezza 3 contenente nella prima componente
%         una variabile intera di scelta tra due condizioni finali:
%
%         se p(1)=1 la spline e' determinata imponendo che la sua 
%         derivata prima calcolata in x(1) e in x(n+1) coincida con 
%         la derivata prima della funzione che si desidera interpolare, 
%         calcolata negli stessi punti; di conseguenza p(2) e p(3)
%         saranno variabili reali contententi tali due derivate;
%
%         se p(1)=2 la spline e' determinata imponendo che la sua derivata
%         seconda calcolata in x(1) e in x(n+1) coincida con con la 
%         derivata seconda della funzione che si desidera interpolare, 
%         calcolata negli stessi punti; di conseguenza p(2) e p(3)
%         saranno variabili reali contententi tali due derivate;
%         (se p(2)=p(3)=0 la spline che si determina e' quella naturale);
%         
%
% Parametri di output:
%_____________________
%
% S       Matrice di dimensione m x 3, contenente, nella riga k-esima,il
%         punto z(k), il valore assunto dalla spline cubica nel 
%         punto z(k) e il valore assunto nello stesso punto dalla 
%         spline derivata.
%
%_____________________________________________________________________________
%
% 1 esempio di utilizzo della function:
% _____________________________________
%
% clear all
% format short e
% 
% 
% alpha=0;beta=2*pi;
% x=[alpha pi/10  pi/7 pi/5 pi/2 pi-pi/9 ...
%    pi-pi/11 pi 3/2*pi-pi/10 3/2*pi-pi/30 beta]';
% 
% f='sin(t)+cos(t)';
% f12=strvcat('cos(t)-sin(t)','-sin(t)-cos(t)');
% 
% t=x;
% y=eval(f);
% y1=eval(f12(1,:));
% y2=eval(f12(2,:));
% n=30;
% h=(beta-alpha)/n;
% z=[alpha:h:beta]';
% 
% p(1)=1;
% 
% t=x(1);
% p(2)=eval(f12(p(1),:));
% 
% t=x(end);
% p(3)=eval(f12(p(1),:));
% 
% S=Spline_interp(x,y,z,p);
% 
%___________________________________________________________________________
%
%
% 2 esempio di utilizzo della function:
% ______________________________________
%
% clear all
% format short e
% 
% 
% alpha=0;beta=2;
% x=[alpha:0.1:beta]';
% 
% f='t.^3+1';
% 
% t=x;
% y=eval(f);
% n=40;
% h=(beta-alpha)/n;
% z=[alpha:h:beta]';
% 
% p(1)=2;
% p(2)=6*x(1);
% p(3)=6*x(end);
% 
% S=Spline_interp(x,y,z,p);
% 
%____________________________________________________________________________



n=length(x)-1;
m=length(z);

h=diff(x);


% Matrice dei coefficienti e termini noti per calcolare M

d1(2:n)=2*(h(1:n-1)+h(2:n));
d2=h;d3=h;

b(1)=(y(2)-y(1))/h(1);
b(2:n)=(y(3:n+1)-y(2:n))./h(2:n)-(y(2:n)-y(1:n-1))./h(1:n-1);
b(n+1)=(y(n+1)-y(n))/h(n);

switch p(1)
case(1)
   d1(1)=2*h(1);
   d1(n+1)=2*h(n);
   
   b(1)=b(1)-p(2);
   b(n+1)=p(3)-b(n+1);
case(2)
   d1=d1(2:n);
   d2=d2(2:n-1);
   d3=d3(2:n-1);
   
   b=b(2:n);
   b(1)=b(1)-h(1)/6*p(2);
   b(n-1)=b(n-1)-h(n)/6*p(3);
   
   Mf(1)=p(2);
   Mf(n+1)=p(3);
 otherwise
   S=[];
   disp('Scelta errata');
   return
end


A=diag(d1)+diag(d3,-1)+diag(d2,+1);
[L,U,P]=lu(A);
yy=L\(P*(6*b)');
M=U\yy;

if p(1)==2
    Mf(2:n)=M;
    M=Mf';
end

% Coefficienti della spline


   c=(y(2:n+1)-y(1:n))./h-h.*(M(2:n+1)-M(1:n))/6;
   d=y(1:n)-h.^2.*M(1:n)/6;
   
       
% Spline cubica interpolante e spline derivata 

z=sort(z);
S=[];
for j=1:m
    xx=z(j);
    for i=1:n
      if xx>=x(i)&xx<=x(i+1)
         s=(M(i+1)*(xx-x(i)).^3-...
            M(i)*(xx-x(i+1)).^3)./(6*h(i))+c(i)*(xx-x(i))+d(i);
         s1=(M(i+1)*(xx-x(i)).^2-...
             M(i)*(xx-x(i+1)).^2)./(2*h(i))+c(i);
                 
      end
      if xx<x(1)
          disp('Un punto e'' esterno all''intervallo');
          return

      end
      if xx>x(n+1)
          disp('Un punto e'' esterno all''intervallo');
          return
      end
    end
    S=[S; s s1];
end
S=[z S];