
function [B,V,it]=Jacobi_soglia(A,nmax,S);
%_____________________________________________________________________________
%
% Function: Jacobi_soglia
% __________________________
%
%
%    Chiamata:  [B,V,it]=Jacobi_soglia(A,nmax,S);
% 
%    Commento:
%
%    La function 'Jacobi_soglia' calcola gli autovalori ed i corrispondenti 
%    autovettori di una matrice A reale e simmetrica mediante il metodo di
%    Jacobi con soglia. 
%    Il metodo azzera gli elementi di B tali che  |Bij|>=S con coppie (i,j)  
%    scelte nell'ordine naturale (Es.(1,2),(1,3),...,(1,n),(2,3)...), ed S  
%    soglia del metodo. S pu essere fissato come parametro di input, oppure, 
%    se S=[], viene ridefinito ad ogni rotazione, nel seguente modo:
%                                          
%                   S=((||Ek||_F)^2/(n*(n-1)))^1/2
%                             
%    dove ||.||_F e' la norma di Frobenius e Ek e' la matrice formata dagli
%    elementi extradiagonali di A al passo k.
%                            
% Parametri di input:
% ____________________
%
% A       matrice reale simmetrica, di dimensione n x n, di cui si cercano 
%         gli autovalori e i corrispondenti autovettori;
% nmax    numero massimo di iterazioni;
% S       soglia del metodo; se S=[] il valore viene calcolato ad ogni 
%         rotazione.
%
% Parametri di output:
% ____________________
%
% B       matrice diagonale, nella tolleranza posta, che contiene le 
%         approssimazioni degli autovalori di A;
% V       matrice le cui colonne contengono gli autovettori corrispondenti
%         agli autovalori;
% it      numero di iterazioni del metodo.
%_____________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% A=[22 -.5 .5; -.5 23 0; .5 0 24.5];
% S=1.e-15;    %oppure S=[]; 
% nmax=60;
% [B,V,it]=Jacobi_soglia(A,nmax,S);
% 
%_____________________________________________________________________________
%

it=[];
B=A;
[n,m]=size(B);
V=eye(n);
if n~=m
    disp('Matrice non quadrata')
    return
end
if A ~= A'
    disp('Matrice non simmetrica')
    return
end
if A==diag(diag(A))
    disp('Gli autovalori sono sulla diagonale di A')
    disp('Gli autovettori sono')
    V
    return
end


flag=0;
if isempty(S)
    S=0.5;
    flag=1;
end
it=0; nrot=1;
while nrot ~= 0    
    nrot=0;
    for p=1:n-1
        for q=p+1:n
            if it==nmax
                disp('** il metodo non converge nel numero di iterazioni fissato **');
                return
            end
            extrad=abs(B(p,q));
            if flag==1
                S=sqrt((norm(B-diag(diag(B)),'fro'))^2/(n*(n-1)));
            end
            if extrad >= S
                mu=(B(q,q)-B(p,p))/(2*B(p,q));
                if mu==0
                    t=1;
                else
                    t=1/(mu+sign(mu)*sqrt(1+mu^2));
                end
                c=1/(sqrt(1+t^2));
                s=c*t;
                Q=eye(n);
                Q(p,p)=c;
                Q(p,q)=s;
                Q(q,p)=-s;
                Q(q,q)=c;
                B=Q'*B*Q;
                V=V*Q;
                it=it+1;
                nrot=nrot+1;
            end % if
        end % q
    end % p
end %while
str=[];
for i=1:n
    str=[str ' %15.10f '];
end
fprintf(' \n Numero di Iterazioni       : %5d \n',it);
fprintf('\n Matrice contenente gli autovalori: \n B = \n');
fprintf( [str '\n'],B');
fprintf('\n Matrice contenente gli autovettori: \n V = \n');
fprintf( [str '\n'],V');            
