
function [lam,xvet,it,lamdiff,xdiff]=Potenze_inv(A,alfa,nmax,toll,x0);
%_____________________________________________________________________________
%
% Function: Potenze_inv
% ______________________
%
%
%
%    Chiamata:  [lam,xvet,it,lamdiff,xdiff]=Potenze_inv(A,alfa,nmax,toll,x0);
% 
%    Commento:
%
%    La function 'Potenze_inv' consente di migliorare l'approssimazione di un 
%    autovalore ottenuta con un altro metodo o di calcolare l'autovalore di modulo  
%    minimo, se si pone alfa=0, di una matrice reale A.
%    L'approssimazione dell'autovalore viene calcolata mediante il quoziente
%    di Rayleigh, e si utilizza la norma due di vettore.
%    La function fornisce anche l'autovettore corrispondente.
%
% Parametri di input:
% ___________________
%
%
% A       matrice di dimensione n x n;
% alfa    parametro di shift che contiene l'approssimazione di un autovalore di A;
%         se alfa=0 calcola l'autovalore di modulo minimo di A;
% nmax    numero massimo di iterazioni;
% toll    tolleranza per il test d'arresto;
% x0      vettore colonna di lunghezza n contenente l'approssimazione iniziale 
%         dell'autovettore.
%
% Parametri di output:
% ____________________
%
%
% lam     vettore colonna  di lunghezza it, contenente le approssimazioni 
%         dell'autovalore cercato (l'ultima componente  la soluzione con la 
%         tolleranza  fissata).
% xvet    matrice, di dimensioni it x n, contenente sulla i-esima riga 
%         l'approssimazione alla iterata i-esima dell'autovettore corrispondente
%         all'autovalore;
% it      numero di iterazioni effettuate dal metodo;
% lamdiff vettore colonna di lunghezza it, contenente il  modulo della differenza
%         lam(i) - lam(i-1) di due approssimazioni successive dell'autovalore; 
% xdiff   vettore colonna di lunghezza it, contenente la norma 2 della differenza
%         xvet(i) - xvet(i-1), di due approssimazioni successive dell'autovettore. 
%        
%
% NB: Si utilizza la decomposizione PA = LU.
%_____________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% Migliorare l'approssimazione degli autovalori di
% A=[4 3 2 1; 3 4 3 2 ; 2 3 4 3; 1 2 3 4]; 
% sapendo che:
%      lam1=11.0990;
%      lam2=3.4142;
%      lam3=0.9010;
%      lam4=0.5858;
% ___________________________________
%   
% A=[4 3 2 1; 3 4 3 2 ; 2 3 4 3; 1 2 3 4]; 
% alfa0=[11.0990 3.4142 0.9010 0.5858];
% nmax=60;
% toll=1.e-14;
% x0=[0 1 0 0]';
% for i=1:length(alfa0)
%     alfa=alfa0(i);
%     [lam,xvet,it,lamdiff,xdiff]=Potenze_inv(A,alfa,nmax,toll,x0);
% end
%___________________________________________________________________________________
%
lam=[];
xvet=[];
lamdiff=[];
xdiff=[];
it=0;
A=A-alfa*eye(size(A));
lam0=alfa;
x0=x0/norm(x0);
[n,m]=size(A);
if n~=m
   disp('***ERRORE*** matrice non quadrata')
   return
end
err=1;
[L,U,P]=lu(A);
while err > toll    & it < nmax 
    pb=P*x0;
    y=L\pb;   % risolve L y  = pb 
    x1=U\y;   % risolve U x1 = y   
    lam1=x0'*x1;
    x1=x1/norm(x1);
    err=abs(lam1-lam0);
    err_x=norm(x1-x0);
    xdiff=[xdiff; err_x];
    lamdiff=[lamdiff;err];
    lam=[lam;1/lam1+alfa];
    xvet=[xvet; x1'];
    x0=x1;
    lam0=lam1;
    it=it+1;
end
fprintf(' \n Numero di Iterazioni       : %5d \n',it);
fprintf(' Autovalore                 : %22.12e \n',lam(it));
fprintf(' Autovettore corrispondente : \n');
disp(x1');
if it==nmax
   disp('** il metodo non converge nel numero di iterazioni fissato **');
end
