
function Ak=QR_shift(A,nmax,toll,n_it);
%_________________________________________________________________________________________
%
% Function: QR_shift
% ___________________
%
%
%
%    Chiamate:  Ak=QR_shift(A,nmax,toll,n_it);
%               Ak=QR_shift(A,nmax,toll);  
% 
%    Commento:
%
%    La function 'QR_shift' implementa il metodo del QR senza shift (per le 
%    prime n_it iterazioni) e con shift (per le successive) per il calcolo degli
%    autovalori della matrice A; lo shift alla (n-k+1)-esima iterazione coincide 
%    con l'elemento B(k,k), dove B  la matrice trasformata dal metodo alla 
%    suddetta iterazione.
%    Per ridurre la complessit computazionale, la matrice A deve essere preventivamente
%    trasformata in una matrice di tipo di Hessenberg superiore mediante il comando
%    MATLAB 'hess(A)'.
%
% Parametri di input:
% ____________________
%
% A       matrice, di dimensione n x n, di cui si vuole conoscere gli autovalori; 
% nmax    numero massimo di iterazioni imposte;
% toll    tolleranza per il test d'arresto;
% n_it    numero di iterazioni che si vuole effettuare con il metodo QR senza shift.
%
% NB:     Se n_it = nmax implementa il metodo del QR senza shift;
%         se n_it = 0, oppure n_it viene omesso nella chiamata, implementa solo il metodo 
%         del QR con shift;
%         se 0 < n_it < nmax il metodo funziona come specificato sopra.
%
% Parametri di output:
% _____________________
%
% Ak      matrice n x n che approssima, nella tolleranza posta e nel caso  
%         di autovalori tutti reali, una matrice triangolare superiore che 
%         contiene, sulla diagonale principale, le approssimazioni degli 
%         autovalori di A. Nel caso in cui la matrice A abbia autovalori 
%         complessi, Ak approssima, nella tolleranza posta, una matrice
%         del tipo triangolare, a blocchi (2 x 2) sulla diagonale. 
%         Gli autovalori complessi coniugati di tali blocchi 2 x 2 
%         sono le approssimazioni degli autovalori complessi di A.
%____________________________________________________________________________________
%
% Esempi di utilizzo della function:
% ___________________________________
%
% shift dall'inizio e per tutte le 
% iterazioni
%
% A1=[4 -1 -1 -1 ; -1 4 -1 -1; -1 -1 4 -1; -1 -1 -1 4 ];
% A=hess(A1);  % trasformazione di A1 in una matrice di tipo Hessenberg superiore
%              % (in questo caso per la simmetria  tridiagonale simmetrica)
% nmax=80;
% toll=1.e-14;
% Ak=QR_shift(A,nmax,toll,0); % Ak=QR_shift(A,nmax,toll); 
% ___________________________________
%
% 3 iterazioni QR senza shift, le 
% successive con shift
%
% A1=[4 -1 -1 -1 ; -1 4 -1 -1; -1 -1 4 -1; -1 -1 -1 4 ];
% A=hess(A1);  % trasformazione di A1 in una matrice di tipo Hessenberg superiore
%              % (in questo caso per la simmetria  tridiagonale simmetrica)
% nmax=80;
% toll=1.e-14;
% n_it=3;
% Ak=QR_shift(A,nmax,toll,n_it);
%___________________________________________________________________________________
%
if nargin==3 | isempty(n_it)
    n_it=0;
end
[n,m]=size(A);
Ak=[];
if n~=m;
    disp('Matrice non quadrata')
    Ak=[];
    return
end
Ak=zeros(n);
B=A;
for k=n:-1:2
    it=0;
    I=eye(k);
    s=abs(B(k,k-1));
    while s > toll*(abs(B(k,k))+abs(B(k-1,k-1))) & it < nmax
       it=it+1;
       if it <= n_it        % metodo QR classico
           [Q,R]=qr(B);
           B=R*Q;
           s=abs(B(k,k-1));
           
       else                 % metodo QR con shift
           mu=B(k,k);  
           [Q,R]=qr(B-mu*I);
           B=R*Q+mu*I; 
           s=abs(B(k,k-1));
       end
    end
    %fprintf('\n%2d Autovalore         : %18.10e \n',n-k+1,B(k,k));
    %fprintf(' Numero di Iterazioni  : %3d \n',it);
    %fprintf(' Elemento extradiag. s : %10.3e\n',s);
    Ak(1:k,k)=B(:,end);
    Ak(k,k-1)=s;
    B=B(1:k-1,1:k-1);
    if it==nmax
        fprintf('Superato in numero max di iteraz. nel calcolo del %2d autovalore \n',n-k+1);
    end
end
Ak(1,1)=B;    
%fprintf('\n%2d Autovalore         : %18.10e \n',n,B);
str=[];
for i=1:n
    str=[str ' %22.10e '];
end
fprintf('\n Matrice contenente gli autovalori: \n Ak = \n');
fprintf( [str '\n'],Ak');
