
function Ak=QR_shift1(A,nmax,toll,n_it);
%_________________________________________________________________________________________
%
% Function: QR_shift1
% ___________________
%
%
%    Chiamate:  Ak=QR_shift1(A,nmax,toll,n_it);
%               Ak=QR_shift1(A,nmax,toll);  
% 
%    Commento:
%
%    La function 'QR_shift1' implementa il metodo del QR senza shift (per le 
%    prime n_it iterazioni) e con shift (per le successive) per il calcolo degli
%    autovalori della matrice A; lo shift alla (n-k+1)-esima iterazione coincide 
%    con l'autovalore del minore:
%
%                |   B(k-1,k-1) B(k-1,k)  |
%                |   B(k,k-1)   B(k,k)    |  k=n,n-1,...,2
%
%    che  pi vicino a B(k,k), dove B  la matrice trasformata dal metodo alla 
%    suddetta iterazione.
%    Questa function cos come  strutturata  efficace per matrici simmetriche,
%    pertanto in questo caso gli autovalori del suddetto minore sono certamente
%    reali. E' conveniente, per ridurre la complessit computazionale, applicare 
%    preventivamente la function hess(A) che in questo caso restituisce una matrice  
%    tridiagonale simmetrica.
%
% Parametri di input:
% ___________________
%
% A       matrice simmetrica, di dimensione n x n, di cui vogliamo conoscere gli
%         autovalori; 
% nmax    numero massimo di iterazioni imposte;
% toll    tolleranza per il test d'arresto;
% n_it    numero di iterazioni che si vuole effettuare con il metodo QR senza shift.
%
% NB:     Se n_it = nmax implementa il metodo del QR senza shift;
%         se n_it = 0, oppure n_it viene omesso nella chiamata, implementa solo il metodo 
%         del QR con shift;
%         se 0 < n_it < nmax il metodo funziona come specificato sopra.
%
% Parametri di output:
% ____________________
%
% Ak      approssima, nella tolleranza posta, una matrice diagonale che contiene
%         le approssimazioni degli autovalori di A.
%
%___________________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% shift dall'inizio e per tutte le 
% iterazioni
%
% A1=[4 -1 -1 -1 ; -1 4 -1 -1; -1 -1 4 -1; -1 -1 -1 4 ];
% A=hess(A1);  %trasformazione di A1 in una matrice tridiagonale
% nmax=40;
% toll=1.e-14;
% Ak=QR_shift1(A,nmax,toll,0);  % Ak=QR_shift1(A,nmax,toll);
% ___________________________________
%
% 3 iterazioni QR senza shift, le 
% successive con shift
%
% A1=[4 -1 -1 -1 ; -1 4 -1 -1; -1 -1 4 -1; -1 -1 -1 4 ];
% A=hess(A1);  %trasformazione di A1 in una matrice tridiagonale
% nmax=40;
% toll=1.e-14;
% n_it=3;
% Ak=QR_shift1(A,nmax,toll,n_it);
%___________________________________________________________________________________
%

if nargin==3 | isempty(n_it)
    n_it=0;
end
[n,m]=size(A);
Ak=[];
if n~=m;
    disp('Matrice non quadrata')
    return
end

if A~=A'
    disp('Matrice non simmetrica')
    return
end

Ak=zeros(n);
B=A;
for k=n:-1:2
    it=0;
    I=eye(k);
    s=abs(B(k,k-1));
    while s > toll*(abs(B(k,k))+abs(B(k-1,k-1))) & it < nmax
       it=it+1;
       if it <= n_it        % metodo QR senza shift
           [Q,R]=qr(B);
           B=R*Q;
           s=abs(B(k,k-1));
           
       else                 % metodo QR con shift
           %calcolo degli autovalori della matrice B(k-1:k,k-1:k)
           b=-trace(B(k-1:k,k-1:k));  c=det(B(k-1:k,k-1:k));
           d=b^2-4*c;
           if b>0
               aut(1)=(-b-sqrt(d))/2;
           else
               aut(1)=(-b+sqrt(d))/2;
           end
           aut(2)=c/aut(1);
           %calcolo dell'autovalore di B(k-1:k,k-1:k) pi vicino a B(k,k)
           v=abs(B(k,k)-aut);
           [Y,ind] = min(v);
           mu=aut(ind);
           [Q,R]=qr(B-mu*I);
           B=R*Q+mu*I;
           s=abs(B(k,k-1));
       end
    end
    fprintf('\n%2d Autovalore         : %18.15e \n',n-k+1,B(k,k));
    fprintf(' Numero di Iterazioni  : %3d \n',it);
    fprintf(' Elemento extradiag. s : %10.3e\n',s);
    Ak(1:k,k)=B(:,end);
    Ak(k,k-1)=s;
    B=B(1:k-1,1:k-1);
    if it==nmax
        fprintf('Superato in numero max di iteraz. nel calcolo del %2d autovalore \n',n-k+1);
    end
end
Ak(1,1)=B;    
fprintf('\n%2d Autovalore         : %18.10e \n',n,B);
str=[];
for i=1:n
    str=[str ' %22.10e '];
end
fprintf('\n Matrice contenente gli autovalori: \n Ak = \n');
fprintf( [str '\n'],Ak');
