
function [xvect,xdiff,fx,it]=Punto_fissoxs(x0,nmax,toll,fun,g);

%_____________________________________________________________________________
%
% Function: Punto_fissoxs
% _______________________
%
%
%    Chiamata:  [xvect,xdiff,fx,it]=Punto_fissoxs(x0,nmax,toll,fun,g);
% 
%    Commento:
%
%    Metodo del punto fisso per la ricerca delle soluzioni di un sistema
%    non lineare di n equazioni in n incognite,con n>=1, 
%    
%    [f1(x);f2(x);...;fn(x)]=[0;0;...;0], (*) 
%    dove x=[x1,x2,...,xn]
%         
%    scritto nella forma equivalente
%
%    [x1;x2;...;xn]=[g1(x);g2(x);...;gn(x)] (**).    
%
%    
%
% Parametri in ingresso:
% ______________________
%
%     x0         Vettore riga di dimensione n, contenente il punto di innesco 
%                del metodo.
%
%     nmax       Variabile intera contenente il numero massimo di iterazioni.
%
%     toll       Variabile reale contenente la tolleranza sul test d'arresto
%                (la differenza tra due iterate, in norma infinito).
%
%     fun        Vettore colonna di dimensione n in cui ogni componente 
%                e' una stringa contenente le espressioni di f1,f2,...,fn
%                in funzione di x.
%                Per creare il vettore fun si puo' usare il comando 'strvcat'
%                nel seguente modo: fun=strvcat(f1,f2,...,fn).
%
%     g          Vettore colonna di lunghezza n in cui ogni componente 
%                e' una stringa contenente le espressioni di g1,g2,...,gn,  
%                in funzione di x, che sono le funzioni di iterazione del
%                metodo; per creare il vettore g si puo' utilizzare il
%                comando 'strvcat' nel seguente modo: g=strvcat(g1,g2,...,gn). 
%       
%                
%
% Parametri in uscita:
% ____________________
%
%     xvect      Matrice di dimensione (it+1) x n contenente, nella k-esima
%                riga, il vettore approssimante la soluzione alla (k-1)-esima
%                iterazione. Pertanto l'ultima riga della matrice contiene il
%                vettore della soluzione approssimata con la tolleranza fissata
%                e la prima riga contiene il vettore d'innesco del metodo.
%
%     xdiff      Vettore colonna di lunghezza it+1, contenente, nella k-esima 
%                componente, k=2,...,it+1, la quantit 
%                norm(xvect(k,:)-xvect(k-1,:),inf); xdiff(1) e' posto 
%                uguale a 0.
%
%     fx         Vettore colonna di lunghezza it+1, contenente, nella k-esima 
%                componente, k=1,...,it+1, la valutazione, in norma infinito,
%                di f in xvect(k,:).
%
%     it         Variabile intera contenente il numero di iterazioni 
%                effettuate dal metodo.
%_____________________________________________________________________________
%
%
%_____________________________________________________________________________
%
%  ESEMPIO
%
%  Dato il sistema non lineare
%	-(1/81)*cos(x1)+(1/9)*x2^2+(1/3)*sin(x3)=x1
%    (1/3)*sin(x1)      +      (1/3)*cos(x3)=x2               
%	-(1/9)*cos(x1) +(1/3)*x2  +(1/6)*sin(x3)=x3
%  verificare che la soluzione sia (0, 1/3, 0), applicando 
%  il metodo del punto fisso al sistema.
%
%   clear all
%   clc
%
% % le componenti di g e di fun vengono inserite come stringhe delle 
% % quali si procede poi alla valutazione con il comando "eval":
%
%   g1='-(1/81)*cos(x(1))+(1/9)*x(2)^2+(1/3)*sin(x(3))';
%   g2=' (1/3)*sin(x(1))+(1/3)*cos(x(3))';
%   g3='-(1/9)*cos(x(1))+(1/3)*x(2)+(1/6)*sin(x(3))';
%   g=strvcat(g1,g2,g3);
%
%   f1='-(1/81)*cos(x(1))+(1/9)*x(2)^2+(1/3)*sin(x(3))-x(1)';
%   f2=' (1/3)*sin(x(1))+(1/3)*cos(x(3))-x(2)';
%	f3='-(1/9)*cos(x(1))+(1/3)*x(2)+(1/6)*sin(x(3))-x(3)';
%   fun=strvcat(f1,f2,f3);
%
% 
%   x0=[0 0 0];
% 	toll=1.e-14;
%	nmax=100;
%	 
%   % chiamata della function
%
%   [xvect,xdiff,fx,it]=Punto_fissoxs(x0,nmax,toll,fun,g);
%_____________________________________________________________________________



n=length(x0);
it=0; diff_x=1;
xvect=x0;
xdiff=0;
x=x0;
for k=1:n
   f(k)=abs(eval(fun(k,:)));
end
fx=norm(f,inf);

while (it<nmax) & (norm(diff_x,inf)>=toll)
   xap=x;
   for k=1:n
     x_new(k)=eval(g(k,:));
%    x(k)=x_new(k); % soluzione in serie (se questa istruzione viene eseguita)
   end
   x=xap;
   xvect=[xvect; x_new];
   diff_x=x-x_new;
   xdiff=[xdiff;norm(diff_x,inf)];
   x=x_new;
   for k=1:n
      res_f(k)=abs(eval(fun(k,:)));
   end
   fx=[fx;norm(res_f,inf)];
   it=it+1;
end

if it>=nmax
    disp('Superato il numero massimo di iterazioni')
end


fprintf(' \n Numero di iterazioni : %5.0f \n',it)
fprintf(' Radice calcolata:\n')
fprintf(' %26.16e  \n', xvect(end,:)')
