
function [T,Y]=Nonlin_ODE(t0,tmax,alfa,beta,N,f,dfi_1,dfi,dfip1,y0);
%________________________________________________________________________
%
% Funzione Nonlin_ODE 
% ____________________
%
% Chiamata:   [T,Y]=Nonlin_ODE(t0,tmax,alfa,beta,N,f,dfi_1,dfi,dfip1,y0);
%
% Commento:
%
% La funzione 'Nonlin_ode' utilizza il metodo di Newton per calcolare la soluzione
% del sistema, in generale, non lineare, ottenuto approssimando con le formule alle
% differenze finite la y'(t) e la y''(t) in un problema ai limiti del 2 ordine della forma:
%
% y''(t)=f(t,y(t),y'(t))     con t in (t0,tmax)
% y(t0)=alfa                                      (*)
% y(tmax)=beta
%
%
% dove f, alfa e beta sono assegnati.
%
% Funzione richiesta: nonlin_matdiff.m 
%
%________________________________________________________________________
% 
%
% Parametri di input della function:
%
% t0,tmax    estremi dell'intervallo [t0,tmax] di integrazione;         
% alfa, beta condizioni ai limiti: y(t0)=alfa, y(tmax)=beta;
% N          numero di sottointervalli in cui e' stato suddiviso [t0,tmax]; 
% f          Stringa contenente l'espressione della f del problema discretizzato:
%            f(t(i),y(i),(y(i+1)-y(i-1))/(2*h));
% dfi_1      stringa contenente l'espressione della derivata della f rispetto 
%            ad y(i-1);
% dfi        stringa contenente l'espressione della derivata della f rispetto 
%            ad y(i);
% dfip1      stringa contenente l'espressione della derivata della f rispetto 
%            ad y(i+1).
% y0         vettore colonna di lunghezza N+1, innesco del metodo di Newton
%            applicato per la risoluzione del sistema ottenuto approssimando 
%            con le formule alle differenze finite la y' e la y'' nel problema 
%            ai limiti (*). Il vettore di innesco y0 va scelto opportunamente
%            in base al sistema ottenuto.Spesso si puo' scegliere y0=zeros(N+1,1).
%
%
% Parametri di output della function:
%
% T          vettore colonna di dimensione N+1 contenente i nodi t0,t1,...,tn;
% Y          vettore colonna di dimensione N+1 contenente i valori della
%            soluzione approssimata, ottenuta dal metodo, nei nodi
%            t0,t1,...,tn.
%_________________________________________________________________________
%
% Esempio  di utilizzo della function  applicata al seguente 
% problema ai limiti:
%
% 
% y''(t)=(32+2*t^3-y(t)*y'(t))/8    t e' in (1,3)
% y(1)=17
% y(3)=43/3
% 
% con soluzione analitica:
% y(t)=t^2+16/t
%
%________________________________________________________________________
%
% clear all; clc
% t0=1;tmax=3;
% alfa=17;
% beta=43/3;
% N=20; 
% y0=zeros(N+1,1);
%
% f='4+1/4*t(i)^3-y(i)*(y(i+1)-y(i-1))/(16*h)';
% dfi_1='y(i)/(16*h)';
% dfi='-(y(i+1)-y(i-1))/(16*h)';
% dfip1='-y(i)/(16*h)';
% 
% %Implementazione del metodo
% 
% [T,Y]=Nonlin_ODE(t0,tmax,alfa,beta,N,f,dfi_1,dfi,dfip1,y0);
% 
%________________________________________________________________________
%
% 
if round(N)==N
h=(tmax-t0)/N;
[fun,Jac]=Nonlin_matdiff(t0,tmax,alfa,beta,N,f,dfi_1,dfi,dfip1);
%
t=(linspace(t0,tmax,N+1))';
nmax=50;  
toll=1.e-14;
%
dim=N+1;
it=1;
res=1;
fxn=zeros(dim,1);
xvect=[];
xdiff=[];
fx=[];
%
while norm(res,inf)>toll
   y=y0;
   for ii=1:dim
      fxn(ii)=eval(fun(ii,:));
      for jj=1:dim
         axn(ii,jj)=eval(Jac((ii-1)*dim+jj,:));
      end
   end
   if abs(det(axn))<=eps
      disp('Il sistema non pu essere risolto: matrice singolare');
      Y=[];T=[];
      return
   end
   res=axn\fxn;  %soluzione del sistema utilizzando il metodo di Gauss
   xdiff=[xdiff;norm(res,inf)];
   y0=y-res;
   xvect=[xvect;y0'];
   fx=[fx;norm(fxn,inf)];
   it=it+1;
   %
   if it>=nmax
      disp('* errore * non converge')
      return;
   end
end
it=it-1;   
Y=xvect(end,:)';
T=t;
else
    disp('N deve essere intero')
    T=[];Y=[];
end
