
function [x,iter,residuo,rho]=Gauss_Seidel_ril(A,b,x0,omega,nmax,toll)

%______________________________________________________________________________
%
% Funzione: Gauss_Seidel_ril
% __________________________
%
%
% Chiamata: [x,iter,residuo,rho]=Gauss_Seidel_ril(A,b,x0,omega,nmax,toll);
%
%
% La function 'Gauss_Seidel_ril' utilizza il metodo iterativo di Gauss-Seidel 
% oppure il metodo di rilassamento successivo (SOR), per la risoluzione di 
% un sistema lineare Ax=b. 
%
% 
% Parametri di ingresso:
% ______________________
%
% A       matrice del sistema di dimensione nxn con gli elementi diagonali 
%         diversi da zero;
% b       vettore colonna di lunghezza n contenente il termine noto del sistema;
% x0      vettore colonna di lunghezza n contenente l'approssimazione iniziale 
%         della soluzione del sistema;
% omega   parametro del metodo di rilassamento (0<omega<2);  ponendo omega=1 
%         si ottiene il metodo di Gauss-Seidel;
% nmax    numero massimo di iterazioni imposte;
% toll    tolleranza per il test d'arresto (sul residuo);
%
%
% Parametri di uscita:
% ____________________
%
% x         matrice (iter+1)xn contenente sulla (i+1)-esima riga l'approssimazione
%           alla iterata i-esima del vettore soluzione del sistema lineare, 
%           i=0,1,...,iter;
% iter      numero di iterazioni effettuate dal metodo per raggiungere la 
%           tolleranza desiderata sul residuo; 
% residuo   vettore colonna di lunghezza iter+1, contenente, nella componente 
%           (i+1)-esima, la norma del residuo alla iterazione i-esima,
%           i=0,1,...,iter;
% rho       raggio spettrale della matrice B di iterazione del metodo.
%
%_______________________________________________________________________________
%
% Esempio di utilizzo della function applicata al sistema lineare Ax=b dove
%
% A=    4 -1  0  0  0
%      -1  4 -1  0  0
%       0 -1  4 -1  0
%       0  0 -1  4 -1
%       0  0  0 -1  4
%
% b=    3
%       2
%       2
%       2
%       3
% 
% con soluzione vera:
%
% alpha= 1
%        1
%        1
%        1
%        1
% _______________________________________________________________________________
% 
% clear all; clc
% n=5;
% A=4*eye(n)-diag(ones(n-1,1),1)-diag(ones(n-1,1),-1)
% b=[3; 2*ones(n-2,1) ; 3]
% x0=zeros(n,1);
% omega=1;      
% nmax=100; toll=1e-12;
%
% [x,iter,residuo,rho]=Gauss_Seidel_ril(A,b,x0,omega,nmax,toll);
%
%________________________________________________________________________________




[n,m]=size(A);
%________________________________________________________________________________

D=diag(diag(A));
OE=omega*tril(A,-1);
B=eye(n)-omega*inv(D+OE)*A;       %matrice di iterazione B del metodo
%________________________________________________________________________________



rho=max(abs(eig(B)));
if rho>=1                   % Condizione necessaria e sufficiente di convergenza 
   disp('*** Il metodo non converge ***')
   iter=0; x=[]; residuo=[];
   return
end



Au=D\triu(A,+1);
Al=D\tril(A,-1);
g=D\b;


xv=x0; xn=x0;
normb=norm(b,inf);
if normb<=eps
   normb=1;
end
res=norm(A*x0-b,inf)/normb;
residuo=res;
x=x0'; i=1;
while res>toll & i<=nmax   
   for j=1:n                 
     xn(j)=(1-omega)*xv(j)+omega*(g(j)-Al(j,:)*xn-Au(j,:)*xv);
   end
   res=norm(A*xn-b,inf)/normb;
   
   x=[x; xn'];
   residuo=[residuo;res];
   xv=xn;
   i=i+1;
end

iter=i-1;
if i>nmax
   disp('** il metodo non converge nel numero di iterazioni fissato **')
end
