
function [x,iter,residuo,rho]=Jacobi_ril(A,b,x0,omega,nmax,toll)

% ______________________________________________________________________________
%
% Funzione: Jacobi_ril
% _____________________
%
%
% Chiamata:   [x,iter,residuo,rho]=Jacobi_ril(A,b,x0,omega,nmax,toll);
%
%
% La function 'Jacobi_ril' utilizza il metodo iterativo di Jacobi oppure il 
% metodo di rilassamento simultaneo (JOR), per la risoluzione di un sistema
% lineare Ax=b. 
%
% 
% Parametri di ingresso:
% ______________________
%
% A       matrice del sistema, di dimensione nxn, con elementi diagonali 
%         diversi da 0;
% b       vettore colonna di lunghezza n contenente il termine noto del sistema;
% x0      vettore colonna di lunghezza n contenente l'approssimazione iniziale 
%         della soluzione del sistema;
% omega   parametro del metodo di rilassamento (il metodo di Jacobi si ottiene 
%         ponendo omega=1);
% nmax    numero massimo di iterazioni imposte;
% toll    tolleranza per il test d'arresto (sul residuo);
%
%
% Parametri di uscita:
% ____________________
%
% x         matrice (iter+1)xn contenente sulla (i+1)-esima riga l'approssimazione
%           alla iterata i-esima del vettore soluzione del sistema lineare, 
%           i=0,1,...,iter;
% iter      numero di iterazioni effettuate dal metodo per raggiungere la 
%           tolleranza desiderata sul residuo; 
% residuo   vettore colonna di lunghezza iter+1, contenente, nella componente 
%           (i+1)-esima, la norma del residuo alla iterazione i-esima,
%           i=0,1,...,iter;
% rho       raggio spettrale della matrice B di iterazione del metodo.
%
%_________________________________________________________________________________
%
% Esempio di utilizzo della function applicata al sistema lineare Ax=b dove
%
% A=    4 -1  0  0  0
%      -1  4 -1  0  0
%       0 -1  4 -1  0
%       0  0 -1  4 -1
%       0  0  0 -1  4
%
% b=    3
%       2
%       2
%       2
%       3
% 
% con soluzione vera:
%
% alpha= 1
%        1
%        1
%        1
%        1
% _______________________________________________________________________________
% 
% clear all; clc
% n=5;
% A=4*eye(n)-diag(ones(n-1,1),1)-diag(ones(n-1,1),-1)
% b=[3; 2*ones(n-2,1) ; 3]
% x0=zeros(n,1);
% omega=1;      
% nmax=100; toll=1e-12;
%
% [x,iter,residuo,rho]=Jacobi_ril(A,b,x0,omega,nmax,toll);
%
%________________________________________________________________________________




[n,m]=size(A);
%___________________________________________________________________

D=diag(diag(A));
B=eye(n)-omega*inv(D)*A;         %matrice di iterazione B del metodo
%___________________________________________________________________

rho=max(abs(eig(B)));
if rho>=1                % Condizione necessaria e sufficiente di convergenza 
   disp('*** Il metodo non converge ***')
   iter=0; x=[]; residuo=[];
   return
end

g=omega*inv(D)*b;                              



xv=x0;
normb=norm(b,inf);
if normb<=eps
   normb=1;
end
res=norm(A*x0-b,inf)/normb;
residuo=res;
x=x0';i=1;
while  res>toll & i<=nmax
   xn=B*xv+g;                   
   x=[x; xn'];
   res=norm(A*xn-b,inf)/normb;
   residuo=[residuo;res];
   
   xv=xn;
   i=i+1;
end

iter=i-1;
if i>nmax
   disp('** il metodo non converge nel numero di iterazioni fissato **');
end
